// 回到顶部组件Func
function scrollIt(destination, duration, easing, callback) {
  // define timing functions -- 过渡动效
  var easings = {
    // no easing, no acceleration
    linear: function (t) {
      return t;
    },
    // accelerating from zero velocity
    easeInQuad: function (t) {
      return t * t;
    },
    // decelerating to zero velocity
    easeOutQuad: function (t) {
      return t * (2 - t);
    },
    // acceleration until halfway, then deceleration
    easeInOutQuad: function (t) {
      return t < 0.5 ? 2 * t * t : -1 + (4 - 2 * t) * t;
    },
    // accelerating from zero velocity
    easeInCubic: function (t) {
      return t * t * t;
    },
    // decelerating to zero velocity
    easeOutCubic: function (t) {
      return --t * t * t + 1;
    },
    // acceleration until halfway, then deceleration
    easeInOutCubic: function (t) {
      return t < 0.5 ? 4 * t * t * t : (t - 1) * (2 * t - 2) * (2 * t - 2) + 1;
    },
    // accelerating from zero velocity
    easeInQuart: function (t) {
      return t * t * t * t;
    },
    // decelerating to zero velocity
    easeOutQuart: function (t) {
      return 1 - --t * t * t * t;
    },
    // acceleration until halfway, then deceleration
    easeInOutQuart: function (t) {
      return t < 0.5 ? 8 * t * t * t * t : 1 - 8 * --t * t * t * t;
    },
    // accelerating from zero velocity
    easeInQuint: function (t) {
      return t * t * t * t * t;
    },
    // decelerating to zero velocity
    easeOutQuint: function (t) {
      return 1 + --t * t * t * t * t;
    },
    // acceleration until halfway, then deceleration
    easeInOutQuint: function (t) {
      return t < 0.5 ? 16 * t * t * t * t * t : 1 + 16 * --t * t * t * t * t;
    }
  };

  (function () {
    var lastTime = 0;
    var vendors = ["ms", "moz", "webkit", "o"];
    for (var x = 0; x < vendors.length && !window.requestAnimationFrame; ++x) {
      window.requestAnimationFrame =
        window[vendors[x] + "RequestAnimationFrame"];
      window.cancelAnimationFrame =
        window[vendors[x] + "CancelAnimationFrame"] ||
        window[vendors[x] + "CancelRequestAnimationFrame"];
    }

    if (!window.requestAnimationFrame)
      window.requestAnimationFrame = function (callback, element) {
        var currTime = new Date().getTime();
        var timeToCall = Math.max(0, 16 - (currTime - lastTime));
        var id = window.setTimeout(function () {
          callback(currTime + timeToCall);
        }, timeToCall);
        lastTime = currTime + timeToCall;
        return id;
      };

    if (!window.cancelAnimationFrame)
      window.cancelAnimationFrame = function (id) {
        clearTimeout(id);
      };
  })();

  function checkElement() {
    // chrome,safari及一些浏览器对于documentElemnt的计算标准化,reset的作用
    document.documentElement.scrollTop += 1;
    var elm =
      document.documentElement.scrollTop !== 0
        ? document.documentElement
        : document.body;
    document.documentElement.scrollTop -= 1;
    return elm;
  }

  var element = checkElement();
  var start = element.scrollTop; // 当前滚动距离
  var startTime = Date.now(); // 当前时间

  function scroll() { // 滚动的实现
    var now = Date.now();
    var time = Math.min(1, (now - startTime) / duration);
    var timeFunction = easings[easing](time);
    element.scrollTop = timeFunction * (destination - start) + start;

    if (element.scrollTop === destination) {
      callback; // 此次执行回调函数
      return;
    }
    window.requestAnimationFrame(scroll);
  }
  scroll();
}

// 回到顶部组件
Vue.component('backtop', {
  props: {
    // 默认在哪个视图显示返回按钮
    pageY: {
      type: Number,
      default: 100
    },
    // 过渡动画名称
    transitionName: {
      type: String,
      default: 'easeInOutQuart'
    }
  },
  data: function () {
    return {
      showTooltips: false,
      showReturnToTop: false
    }
  },
  methods: {
    currentPageYOffset: function () {
      window.pageYOffset > this.pageY ? this.showReturnToTop = true : this.showReturnToTop = false
    },
    backToTop: function () {
      scrollIt(0, 600, this.transitionName, this.currentPageYOffset);
    },
  },
  template: "<p class='backTop' @click='backToTop' v-show='showReturnToTop' style='z-index: 100000000;bottom: 60px;'><span class='icon iconfont icon-4fanhuidingbucheng' style='color: #fff !important;font-size: 25px;'></span></p>",
  created: function () {
    window.addEventListener('scroll', this.currentPageYOffset);
  },
  beforeDestroy: function () {
    window.removeEventListener('scroll', this.currentPageYOffset)
  },

})